/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __SyncGenie_CPP__
#define   __SyncGenie_CPP__

#include "SyncGenie.h"


// SyncGenie::SyncGenie() : lcd( LCD_RS, LCD_EN, LCD_D4, LCD_D5, LCD_D6, LCD_D7 ), pushbutton( BUTTON ) { } // moved to header

SyncGenie::~SyncGenie()
{
	
}

void
SyncGenie::begin( void )
{
	this->begin( "Teensy @TEENSY_VERSION@ @TEENSY_LENGTH@\nGenie Rev @BOARD_REVISION@" );
}

void
SyncGenie::begin( const String & msg )
{
	Serial.begin( 115200 );
	// For the following pins, the only modes that make sense at all are:
	if( POT1 >= 0 ) pinMode( POT1,   INPUT  );
	if( POT2 >= 0 ) pinMode( POT2,   INPUT  );
	//if( BUTTON >= 0         ) pinMode( BUTTON,         INPUT_PULLUP ); // this is handled by the constructor of member Switch instance .pushButton
	//if( ENCODER_BUTTON >= 0 ) pinMode( ENCODER_BUTTON, INPUT_PULLUP ); // this is handled by the constructor of member Switch instance .selectorButton
	//if( ENCODER_A >= 0      ) pinMode( ENCODER_A,      INPUT_PULLUP ); // this is handled by the constructor of member Wheel instance .selectorWheel
	//if( ENCODER_B >= 0      ) pinMode( ENCODER_B,      INPUT_PULLUP ); // this is handled by the constructor of member Wheel instance .selectorWheel
	this->pushButton.begin();
	this->selectorButton.begin();
	this->selectorWheel.begin();
	if( D8_LED >= 0 ) { pinMode( D8_LED, OUTPUT ); digitalWrite( D8_LED, LOW ); } // LED_BUILTIN will be one of these
	if( D9_LED >= 0 ) { pinMode( D9_LED, OUTPUT ); digitalWrite( D9_LED, LOW ); } // LED_BUILTIN will be one of these
	// The following modes *could* be changed but in 99% of cases you're going to use them as:
	if( D0_OUT >= 0 ) { pinMode( D0_OUT, OUTPUT ); digitalWrite( D0_OUT, LOW ); }
	if( D1_OUT >= 0 ) { pinMode( D1_OUT, OUTPUT ); digitalWrite( D1_OUT, LOW ); }
	if( D2_OUT >= 0 ) { pinMode( D2_OUT, OUTPUT ); digitalWrite( D2_OUT, LOW ); }
	if( D3_OUT >= 0 ) { pinMode( D3_OUT, OUTPUT ); digitalWrite( D3_OUT, LOW ); }
	if( D4_OUT >= 0 ) { pinMode( D4_OUT, OUTPUT ); digitalWrite( D4_OUT, LOW ); }
	if( D5_OUT >= 0 ) { pinMode( D5_OUT, OUTPUT ); digitalWrite( D5_OUT, LOW ); }
	if( D6_OUT >= 0 ) { pinMode( D6_OUT, OUTPUT ); digitalWrite( D6_OUT, LOW ); }
	if( D7_OUT >= 0 ) { pinMode( D7_OUT, OUTPUT ); digitalWrite( D7_OUT, LOW ); }
	// The following modes *could* be changed but in 99% of cases you're going to use them as:
	if( D0_IN >= 0 ) pinMode( D0_IN,  INPUT  );
	if( D1_IN >= 0 ) pinMode( D1_IN,  INPUT  );
	if( D2_IN >= 0 ) pinMode( D2_IN,  INPUT  );
	if( D3_IN >= 0 ) pinMode( D3_IN,  INPUT  );
	if( D4_IN >= 0 ) pinMode( D4_IN,  INPUT  );
	if( D5_IN >= 0 ) pinMode( D5_IN,  INPUT  );
	if( D6_IN >= 0 ) pinMode( D6_IN,  INPUT  );
	if( D7_IN >= 0 ) pinMode( D7_IN,  INPUT  );
	// The following modes only apply to the SyncGenie revision 2 (a.k.a. the Ortiz Box) which had an RJ45 connector with pins 1 & 2 available and pins 7 & 8 grounded:
	if( RJ45_PIN1 >= 0 ) pinMode( RJ45_PIN1, INPUT_PULLUP );
	if( RJ45_PIN2 >= 0 ) pinMode( RJ45_PIN2, INPUT_PULLUP );
	
	// TRS and BNC modes will be left undefined (so on a Teensy that means the corresponding pins will be disabled until you explicitly set their mode)
	// I2C will not be started by default - you will have to do the whole Wire.setClock() and Wire.begin() thing yourself.
	
	if( LCD_COLS > 0 && LCD_ROWS > 0 ) this->lcd.begin( LCD_COLS, LCD_ROWS );
	this->display( msg );
	this->flicker();
}

void
SyncGenie::flicker( int ledPin, unsigned long millisOn, unsigned long millisOff, unsigned long millisTotal )
{
	if( ledPin < 0 ) return;
	unsigned long startTime = millis();
	pinMode( ledPin, OUTPUT );
	for( unsigned long i = 0; millis() - startTime < millisTotal; i++ )
	{
		digitalWrite( ledPin, i % 2 ? LOW : HIGH );
		delay( i % 2 ? millisOff : millisOn );
	}
	digitalWrite( ledPin, LOW );
}

void
SyncGenie::display( const String & msg, int row, bool echoToSerial )
{
	if( !msg.length() ) return;
	bool doLCD = ( this->LCD_COLS > 0 && this->LCD_ROWS > 0 );
	if( doLCD ) this->lcd.setCursor( 0, row );
	if( doLCD || echoToSerial )
	{
		for( unsigned int i = 0; i < msg.length(); i++ )
		{
			if( msg[ i ] == '\f' )
			{
				if( doLCD ) this->lcd.clear();
				if( echoToSerial ) Serial.print( '\n' );
				continue;
			}
			if( msg[ i ] == '\n' )
			{
				if( doLCD )
				{
					for( int column = 0; column < LCD_COLS; column++ ) this->lcd.print( ' ' );
					this->lcd.setCursor( 0, ++row );
				}
				if( echoToSerial ) Serial.print( '\n' );
				continue;
			}
			if( msg[ i ] == '@' )
			{
				unsigned int n;
#				define INTERPOLATE( VARNAME, VALUE )    if( 0 == ::strncmp( msg.c_str() + i + 1, VARNAME "@", ( n = ::strlen( VARNAME "@" ) ) ) ) { i += n; if( doLCD ) this->lcd.print( VALUE ); if( echoToSerial ) Serial.print( VALUE ); continue; }
				INTERPOLATE( "TEENSY_VERSION", TEENSY_VERSION );
				INTERPOLATE( "TEENSY_LENGTH",  TEENSY_LONG_SHORT( "LONG", "SHORT" ) );
				INTERPOLATE( "BOARD_REVISION", this->BOARD_REVISION );
			}
			if( doLCD ) this->lcd.print( msg[ i ] );
			if( echoToSerial ) Serial.print( msg[ i ] );
		}
	}
	if( doLCD ) for( int column = 0; column < LCD_COLS; column++ ) this->lcd.print( ' ' );
	if( echoToSerial ) Serial.print( '\n' );
}

int
SyncGenie::bar( int height )
{
	if( this->D0_OUT >= 0 ) digitalWrite( this->D0_OUT, height >=  1 );
	if( this->D1_OUT >= 0 ) digitalWrite( this->D1_OUT, height >=  2 );
	if( this->D2_OUT >= 0 ) digitalWrite( this->D2_OUT, height >=  3 );
	if( this->D3_OUT >= 0 ) digitalWrite( this->D3_OUT, height >=  4 );
	if( this->D4_OUT >= 0 ) digitalWrite( this->D4_OUT, height >=  5 );
	if( this->D5_OUT >= 0 ) digitalWrite( this->D5_OUT, height >=  6 );
	if( this->D6_OUT >= 0 ) digitalWrite( this->D6_OUT, height >=  7 );
	if( this->D7_OUT >= 0 ) digitalWrite( this->D7_OUT, height >=  8 );
	if( this->D8_LED >= 0 ) digitalWrite( this->D8_LED, height >=  9 );
	if( this->D9_LED >= 0 ) digitalWrite( this->D9_LED, height >= 10 );
	return height;
}

float
SyncGenie::temperature( void )
{
#	if defined(TEENSYDUINO) && !defined(ARDUINO_TEENSY30) && !defined(ARDUINO_TEENSY31) && !defined(ARDUINO_TEENSYLC) && !defined(ARDUINO_TEENSY32) && !defined(ARDUINO_TEENSY35) && !defined(ARDUINO_TEENSY36)
		return tempmonGetTemp();
#	else
		return 0.0; // there is a long-winded way to do it on Teensy 3.x, but the calibration reportedly varies a lot so it may not be worth the effort
#	endif 
}


void
SyncGenie::loadMessage( const char * msg )
{
	mMessageStart = mMessageCursor = msg;
}

void
SyncGenie::showMessage( void )
{
	const char * cursor = mMessageCursor;
	for( int iRow = 0; iRow < this->LCD_ROWS; iRow++ )
		cursor = this->messageLine( cursor, iRow );
}

void
SyncGenie::scrollMessage( int lines )
{
	if( lines > 0 )
	{
		for( int i = 0; i < lines; i++ )
		{
			const char * next = this->messageLine( mMessageCursor );
			if( *next ) mMessageCursor = next;
		}
	}
	if( lines < 0 )
	{
		for( int i = 0; i < -lines; i++ )
		{
			const char * start = mMessageStart;
			while( true )
			{
				const char * stop = this->messageLine( start );
				if( stop == mMessageCursor ) { mMessageCursor = start; break; }
				else if( *stop ) start = stop;
				else break;
			}
		}
	}
	this->showMessage();
}

const char *
SyncGenie::messageLine( const char * start, int lcdRow )
{
	while( *start && isspace( *start ) ) start++;
	const char * stop = start, * breakpoint = NULL;
	while( true )
	{
		if( stop > start && ( !*stop || isspace( *stop ) ) ) breakpoint = stop;
		if( !*stop || *stop == '\n' || stop == start + this->LCD_COLS ) break;
		stop++;
	}
	if( *stop && !isspace( *stop ) && breakpoint ) stop = breakpoint;
	if( lcdRow >= 0 )
	{
		this->lcd.setCursor( 0, lcdRow );
		for( int iCol = 0; iCol < this->LCD_COLS; iCol++ )
		{
			if( start < stop && *start == '\t' ) start++;
			this->lcd.print( ( start < stop ) ? *start++ : ' ' );
		}
	}
	return stop;
}

#endif // __SyncGenie_CPP__
