/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __SyncGeniePins_H__
#define   __SyncGeniePins_H__

#include "SyncGenieTeensyVersioning.h"

static const int BOARD_REVISION = 4;
// The first build was on 2024-07-01. The revision number is printed on the board.

static const int D0_OUT    = 7;
static const int D1_OUT    = 6;
static const int D2_OUT    = 5;
static const int D3_OUT    = 4;
static const int D4_OUT    = 3;
static const int D5_OUT    = 2;
static const int D6_OUT    = 1;
static const int D7_OUT    = 0;

static const int D8_LED    = TEENSY_LONG_ONLY( 32 );
static const int D9_LED    = 13;

#if defined(ARDUINO_TEENSY36) || defined(ARDUINO_TEENSY35)
static const int D7_IN     = TEENSY_LONG_ONLY( 67 ); // On the Teensy 3.6 and 3.5 diagrams, this physical position is marked "A22/DAC1" without a pin number (pin 41 is on the back). The board library headers define A22 as 67.
static const int D6_IN     = TEENSY_LONG_ONLY( 66 ); // On the Teensy 3.6 and 3.5 diagrams, this physical position is marked "A21/DAC0" without a pin number (pin 40 is on the back). The board library headers define A21 as 66.
#else
static const int D7_IN     = TEENSY_LONG_ONLY( 41 ); // On the Teensy 4.1 diagram, this physical position is marked with pin number 41, continuing the front-side numbering as you would expect.
static const int D6_IN     = TEENSY_LONG_ONLY( 40 ); // On the Teensy 4.1 diagram, this physical position is marked with pin number 40, continuing the front-side numbering as you would expect.
#endif
static const int D5_IN     = TEENSY_LONG_ONLY( 39 );
static const int D4_IN     = TEENSY_LONG_ONLY( 38 );
static const int D3_IN     = TEENSY_LONG_ONLY( 37 );
static const int D2_IN     = TEENSY_LONG_ONLY( 36 );
static const int D1_IN     = TEENSY_LONG_ONLY( 35 );
static const int D0_IN     = TEENSY_LONG_ONLY( 34 );

static const int RJ45_PIN1 = -1; // only the SyncGenie revision 2 (a.k.a. the Ortiz Box) had this connector
static const int RJ45_PIN2 = -1; // only the SyncGenie revision 2 (a.k.a. the Ortiz Box) had this connector

static const int SELECTOR_BUTTON =  8;
static const int SELECTOR_A      =  9;
static const int SELECTOR_B      = 10;
static const int BNC1            = 12; // left  when bargraph LEDS are on top
static const int BNC2            = 11; // right when bargraph LEDS are on top
static const int POT1            = 17; // left  when bargraph LEDS are on top
static const int POT2            = 16; // right when bargraph LEDS are on top
static const int JST_PIN3        = 18; // second-from-right pin when bargraph LEDs are on top. Used as SDA when plugging into an I2C device (JST pins from left: GND, 3.3V, SDA, SCL) but may be turned to other purposes.
static const int JST_PIN4        = 19; //         rightmost pin when bargraph LEDs are on top. Used as SCL when plugging into an I2C device (JST pins from left: GND, 3.3V, SDA, SCL) but may be turned to other purposes.
static const int TRS1_TIP        = 22;
static const int TRS1_RING       = 23;
static const int TRS2_TIP        = 20;
static const int TRS2_RING       = 21;
static const int TRS3_TIP        = 14;
static const int TRS3_RING       = 15;
static const int TRS4_TIP        = TEENSY_LONG_ONLY( 24 );
static const int TRS4_RING       = TEENSY_LONG_ONLY( 25 );
static const int LCD_RS          = TEENSY_LONG_ONLY( 26 );
static const int LCD_EN          = TEENSY_LONG_ONLY( 27 );
static const int LCD_D4          = TEENSY_LONG_ONLY( 28 );
static const int LCD_D5          = TEENSY_LONG_ONLY( 29 );
static const int LCD_D6          = TEENSY_LONG_ONLY( 30 );
static const int LCD_D7          = TEENSY_LONG_ONLY( 31 );
static const int BUTTON          = TEENSY_LONG_ONLY( 33 );

#ifdef __FakeLiquidCrystal_H__
	static const int LCD_COLS        = 16;
	static const int LCD_ROWS        =  2;
#else
	static const int LCD_COLS        = TEENSY_LONG_ONLY( 16 );
	static const int LCD_ROWS        = TEENSY_LONG_ONLY(  2 );
#endif

#endif // __SyncGeniePins_H__
